from aiopoke.objects.resources.encounters.encounter_condition import EncounterCondition
from aiopoke.objects.resources.encounters.encounter_condition_value import (
    EncounterConditionValue,
)
from aiopoke.objects.resources.encounters.encounter_method import EncounterMethod

__all__ = ("EncounterCondition", "EncounterConditionValue", "EncounterMethod")



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility import FlavorText
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources.moves import Move


class SuperContestEffect(Resource):
    appeal: int
    flavor_text_entries: List["FlavorText"]
    id: int
    moves: List[MinimalResource["Move"]]

    def __init__(
        self,
        *,
        id: int,
        appeal: int,
        flavor_text_entries: List[Dict[str, Any]],
        moves: List[Dict[str, Any]]
    ) -> None:
        self.id = id
        self.appeal = appeal
        self.flavor_text_entries = [
            FlavorText(**flavor_text_entry) for flavor_text_entry in flavor_text_entries
        ]
        self.moves = [MinimalResource(**move) for move in moves]



from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import TYPE_CHECKING

from aiopoke.objects.utility import Name
from aiopoke.objects.utility import NamedResource
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.minimal_resources import Url
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import Move, MoveDamageClass
    from aiopoke.objects.resources.pokemon import Characteristic, Nature


class Stat(NamedResource):
    affecting_moves: "MoveStatAffectSets"
    affecting_natures: "NatureStatAffectSets"
    characteristics: List[Url["Characteristic"]]
    game_index: int
    is_battle_only: bool
    move_damage_class: Optional[MinimalResource["MoveDamageClass"]]
    names: List["Name"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        affecting_moves: Dict[str, Any],
        affecting_natures: Dict[str, Any],
        characteristics: List[Dict[str, Any]],
        game_index: int,
        is_battle_only: bool,
        move_damage_class: Optional[Dict[str, Any]],
        names: List[Dict[str, Any]],
    ):
        super().__init__(id=id, name=name)
        self.affecting_moves = MoveStatAffectSets(**affecting_moves)
        self.affecting_natures = NatureStatAffectSets(**affecting_natures)
        self.characteristics = [
            Url(**characteristic) for characteristic in characteristics
        ]
        self.game_index = game_index
        self.is_battle_only = is_battle_only
        self.move_damage_class = (
            MinimalResource(**move_damage_class)
            if move_damage_class is not None
            else None
        )
        self.names = [Name(**name) for name in names]


class MoveStatAffectSets(Resource):
    increase: List["MoveStatAffect"]
    decrease: List["MoveStatAffect"]

    def __init__(
        self,
        *,
        increase: List[Dict[str, Any]],
        decrease: List[Dict[str, Any]],
    ) -> None:
        self.increase = [
            MoveStatAffect(**move_stat_affect) for move_stat_affect in increase
        ]
        self.decrease = [
            MoveStatAffect(**move_stat_affect) for move_stat_affect in decrease
        ]


class MoveStatAffect(Resource):
    change: int
    move: MinimalResource["Move"]

    def __init__(
        self,
        *,
        change: int,
        move: Dict[str, Any],
    ) -> None:
        self.change = change
        self.move = MinimalResource(**move)


class NatureStatAffectSets(Resource):
    increase: List[MinimalResource["Nature"]]
    decrease: List[MinimalResource["Nature"]]

    def __init__(
        self, *, increase: List[Dict[str, Any]], decrease: List[Dict[str, Any]]
    ) -> None:
        self.increase = [MinimalResource(**nature_data) for nature_data in increase]
        self.decrease = [MinimalResource(**nature_data) for nature_data in decrease]



from typing import Any, Dict, List

from aiopoke.objects.utility import Effect, FlavorText
from aiopoke.utils.resource import Resource


class ContestEffect(Resource):
    appeal: int
    effect_entries: List["Effect"]
    flavor_text_entries: List["FlavorText"]
    id: int
    jam: int

    def __init__(
        self,
        *,
        id: int,
        appeal: int,
        effect_entries: List[Dict[str, Any]],
        flavor_text_entries: List[Dict[str, Any]],
        jam: int
    ) -> None:
        self.id = id
        self.appeal = appeal
        self.effect_entries = [Effect(**effect) for effect in effect_entries]
        self.flavor_text_entries = [
            FlavorText(**flavor) for flavor in flavor_text_entries
        ]
        self.jam = jam



from aiopoke.objects.utility.common_models import Description
from aiopoke.objects.utility.common_models import Effect
from aiopoke.objects.utility.common_models import Encounter
from aiopoke.objects.utility.common_models import FlavorText
from aiopoke.objects.utility.common_models import GenerationGameIndex
from aiopoke.objects.utility.common_models import MachineVersionDetail
from aiopoke.objects.utility.common_models import Name
from aiopoke.objects.utility.common_models import NamedResource
from aiopoke.objects.utility.common_models import Sprite
from aiopoke.objects.utility.common_models import Sprites
from aiopoke.objects.utility.common_models import VerboseEffect
from aiopoke.objects.utility.common_models import VersionEncounterDetail
from aiopoke.objects.utility.common_models import VersionGameIndex
from aiopoke.objects.utility.common_models import VersionGroupFlavorText
from aiopoke.objects.utility.language import Language

__all__ = (
    "Language",
    "Description",
    "Effect",
    "Encounter",
    "FlavorText",
    "GenerationGameIndex",
    "MachineVersionDetail",
    "Name",
    "NamedResource",
    "Sprite",
    "Sprites",
    "VerboseEffect",
    "VersionEncounterDetail",
    "VersionGameIndex",
    "VersionGroupFlavorText",
)



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility import Name, NamedResource
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources.berries import Berry
    from aiopoke.objects.resources.contests.contest_type import ContestType


class BerryFlavor(NamedResource):
    berries: List["FlavorBerryMap"]
    contest_type: MinimalResource["ContestType"]
    names: List["Name"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        berries: List[Dict[str, Any]],
        contest_type: Dict[str, Any],
        names: List[Dict[str, Any]]
    ) -> None:
        super().__init__(id=id, name=name)
        self.berries = [FlavorBerryMap(**berry) for berry in berries]
        self.contest_type = MinimalResource(**contest_type)
        self.names = [Name(**name) for name in names]


class FlavorBerryMap(Resource):
    potency: int
    berry: MinimalResource["Berry"]

    def __init__(self, *, potency: int, berry: Dict[str, Any]) -> None:
        self.potency = potency
        self.berry = MinimalResource(**berry)



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility.common_models import Name, NamedResource
from aiopoke.utils.minimal_resources import MinimalResource

if TYPE_CHECKING:
    from aiopoke.objects.resources.pokemon import PokemonSpecies


class PokemonColor(NamedResource):
    names: List["Name"]
    pokemon_species: List[MinimalResource["PokemonSpecies"]]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        names: List[Dict[str, Any]],
        pokemon_species: List[Dict[str, Any]]
    ) -> None:
        super().__init__(id=id, name=name)
        self.names = [Name(**name) for name in names]
        self.pokemon_species = [
            MinimalResource(**pokemon_species) for pokemon_species in pokemon_species
        ]



from typing import TYPE_CHECKING, Any, Dict

from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import Generation


class GenerationGameIndex(Resource):
    game_index: int
    generation: MinimalResource["Generation"]

    def __init__(self, *, game_index: int, generation: Dict[str, Any]) -> None:
        self.game_index = game_index
        self.generation = MinimalResource(**generation)



from aiopoke.objects.resources.contests.contest_effect import ContestEffect
from aiopoke.objects.resources.contests.contest_type import ContestType
from aiopoke.objects.resources.contests.super_contest_effect import SuperContestEffect

__all__ = ("ContestEffect", "ContestType", "SuperContestEffect")



from aiopoke.objects.resources.pokemon.ability import Ability
from aiopoke.objects.resources.pokemon.characteristic import Characteristic
from aiopoke.objects.resources.pokemon.egg_group import EggGroup
from aiopoke.objects.resources.pokemon.gender import Gender
from aiopoke.objects.resources.pokemon.growth_rate import GrowthRate
from aiopoke.objects.resources.pokemon.natural_gift_type import NaturalGiftType
from aiopoke.objects.resources.pokemon.nature import Nature
from aiopoke.objects.resources.pokemon.pokeathlon_stat import PokeathlonStat
from aiopoke.objects.resources.pokemon.pokemon import Pokemon
from aiopoke.objects.resources.pokemon.pokemon_color import PokemonColor
from aiopoke.objects.resources.pokemon.pokemon_form import PokemonForm
from aiopoke.objects.resources.pokemon.pokemon_habitat import PokemonHabitat
from aiopoke.objects.resources.pokemon.pokemon_shape import PokemonShape
from aiopoke.objects.resources.pokemon.pokemon_species import PokemonSpecies
from aiopoke.objects.resources.pokemon.stat import Stat

__all__ = (
    "Ability",
    "Characteristic",
    "EggGroup",
    "Gender",
    "GrowthRate",
    "NaturalGiftType",
    "Nature",
    "PokeathlonStat",
    "PokemonColor",
    "PokemonForm",
    "PokemonHabitat",
    "PokemonShape",
    "PokemonSpecies",
    "Pokemon",
    "Stat",
)



from typing import Union

from aiopoke.http_client import HttpClient
from aiopoke.objects.resources import Ability
from aiopoke.objects.resources import Berry
from aiopoke.objects.resources import BerryFirmness
from aiopoke.objects.resources import BerryFlavor
from aiopoke.objects.resources import Characteristic
from aiopoke.objects.resources import ContestEffect
from aiopoke.objects.resources import ContestType
from aiopoke.objects.resources import EggGroup
from aiopoke.objects.resources import EncounterCondition
from aiopoke.objects.resources import EncounterConditionValue
from aiopoke.objects.resources import EncounterMethod
from aiopoke.objects.resources import EvolutionChain
from aiopoke.objects.resources import EvolutionTrigger
from aiopoke.objects.resources import Gender
from aiopoke.objects.resources import Generation
from aiopoke.objects.resources import GrowthRate
from aiopoke.objects.resources import Item
from aiopoke.objects.resources import ItemAttribute
from aiopoke.objects.resources import ItemCategory
from aiopoke.objects.resources import ItemFlingEffect
from aiopoke.objects.resources import ItemPocket
from aiopoke.objects.resources import Location
from aiopoke.objects.resources import LocationArea
from aiopoke.objects.resources import Machine
from aiopoke.objects.resources import Move
from aiopoke.objects.resources import MoveAilment
from aiopoke.objects.resources import MoveBatteStyle
from aiopoke.objects.resources import MoveCategory
from aiopoke.objects.resources import MoveDamageClass
from aiopoke.objects.resources import MoveLearnMethod
from aiopoke.objects.resources import MoveTarget
from aiopoke.objects.resources import NaturalGiftType
from aiopoke.objects.resources import Nature
from aiopoke.objects.resources import PalParkArea
from aiopoke.objects.resources import PokeathlonStat
from aiopoke.objects.resources import Pokedex
from aiopoke.objects.resources import Pokemon
from aiopoke.objects.resources import PokemonColor
from aiopoke.objects.resources import PokemonForm
from aiopoke.objects.resources import PokemonHabitat
from aiopoke.objects.resources import PokemonShape
from aiopoke.objects.resources import PokemonSpecies
from aiopoke.objects.resources import Region
from aiopoke.objects.resources import Stat
from aiopoke.objects.resources import SuperContestEffect
from aiopoke.objects.resources import Version
from aiopoke.objects.resources import VersionGroup
from aiopoke.objects.utility import Sprite
from aiopoke.objects.utility.language import Language
from aiopoke.utils import Cache
from aiopoke.utils import cache
from aiopoke.utils import Url


class MetaClient(type):
    def __new__(cls, name, bases, clsdict):
        for key, value in clsdict.items():
            if not hasattr(value, "__qualname__"):
                continue

            if key.startswith("get") and value.__qualname__.startswith("AiopokeClient"):
                endpoint = value.__name__.split("get_")[1].replace("_", "-")
                clsdict[key] = cache(endpoint, value)

        return super().__new__(cls, name, bases, clsdict)


class AiopokeClient(metaclass=MetaClient):
    http: HttpClient
    _cache: Cache

    def __init__(self, *, session=None) -> None:
        self.http = HttpClient(session=session)
        self._cache = Cache()
        Url.link(self)
        Sprite.link(self)

    async def __aenter__(self):
        return self

    async def __aexit__(self, *args):
        await self.close()

    async def close(self):
        await self.http.close()

    async def get_ability(self, name_or_id: Union[str, int]) -> Ability:
        data = await self.http.get(f"ability/{name_or_id}")
        return Ability(**data)

    async def get_berry(self, name_or_id: Union[str, int]) -> Berry:
        data = await self.http.get(f"berry/{name_or_id}")
        return Berry(**data)

    async def get_berry_flavor(self, name_or_id: Union[str, int]) -> BerryFlavor:
        data = await self.http.get(f"berry-flavor/{name_or_id}")
        return BerryFlavor(**data)

    async def get_berry_firmness(self, name_or_id: Union[str, int]) -> BerryFirmness:
        data = await self.http.get(f"berry-firmness/{name_or_id}")
        return BerryFirmness(**data)

    async def get_characteristic(self, name_or_id: Union[str, int]) -> Characteristic:
        data = await self.http.get(f"characteristic/{name_or_id}")
        return Characteristic(**data)

    async def get_contest_effect(self, id: int) -> ContestEffect:
        data = await self.http.get(f"contest-effect/{id}")
        return ContestEffect(**data)

    async def get_contest_type(self, name_or_id: Union[str, int]) -> ContestType:
        data = await self.http.get(f"contest-type/{name_or_id}")
        return ContestType(**data)

    async def get_egg_group(self, name_or_id: Union[str, int]) -> EggGroup:
        data = await self.http.get(f"egg-group/{name_or_id}")
        return EggGroup(**data)

    async def get_encounter_condition(
        self, name_or_id: Union[str, int]
    ) -> EncounterCondition:
        data = await self.http.get(f"encounter-condition/{name_or_id}")
        return EncounterCondition(**data)

    async def get_encounter_condition_value(
        self, name_or_id
    ) -> EncounterConditionValue:
        data = await self.http.get(f"encounter-condition-value/{name_or_id}")
        return EncounterConditionValue(**data)

    async def get_encounter_method(
        self, name_or_id: Union[str, int]
    ) -> EncounterMethod:
        data = await self.http.get(f"encounter-method/{name_or_id}")
        return EncounterMethod(**data)

    async def get_evolution_chain(self, id: int) -> EvolutionChain:
        data = await self.http.get(f"evolution-chain/{id}")
        return EvolutionChain(**data)

    async def get_evolution_trigger(self, id: int) -> EvolutionTrigger:
        data = await self.http.get(f"evolution-trigger/{id}")
        return EvolutionTrigger(**data)

    async def get_gender(self, name_or_id: Union[str, int]) -> Gender:
        data = await self.http.get(f"gender/{name_or_id}")
        return Gender(**data)

    async def get_generation(self, name_or_id: Union[str, int]) -> Generation:
        data = await self.http.get(f"generation/{name_or_id}")
        return Generation(**data)

    async def get_growth_rate(self, name_or_id: Union[str, int]) -> GrowthRate:
        data = await self.http.get(f"growth-rate/{name_or_id}")
        return GrowthRate(**data)

    async def get_item(self, name_or_id: Union[str, int]) -> Item:
        data = await self.http.get(f"item/{name_or_id}")
        return Item(**data)

    async def get_item_attribute(self, name_or_id: Union[str, int]) -> ItemAttribute:
        data = await self.http.get(f"item-attribute/{name_or_id}")
        return ItemAttribute(**data)

    async def get_item_category(self, name_or_id: Union[str, int]) -> ItemCategory:
        data = await self.http.get(f"item-category/{name_or_id}")
        return ItemCategory(**data)

    async def get_item_fling_effect(
        self, name_or_id: Union[str, int]
    ) -> ItemFlingEffect:
        data = await self.http.get(f"item-fling-effect/{name_or_id}")
        return ItemFlingEffect(**data)

    async def get_item_pocket(self, name_or_id: Union[str, int]) -> ItemPocket:
        data = await self.http.get(f"item-pocket/{name_or_id}")
        return ItemPocket(**data)

    async def get_language(self, name_or_id: Union[str, int]) -> Language:
        data = await self.http.get(f"language/{name_or_id}")
        return Language(**data)

    async def get_location(self, name_or_id: Union[str, int]) -> Location:
        data = await self.http.get(f"location/{name_or_id}")
        return Location(**data)

    async def get_location_area(self, name_or_id: Union[str, int]) -> LocationArea:
        data = await self.http.get(f"location-area/{name_or_id}")
        return LocationArea(**data)

    async def get_machine(self, name_or_id: Union[str, int]) -> Machine:
        data = await self.http.get(f"machine/{name_or_id}")
        return Machine(**data)

    async def get_move(self, name_or_id: Union[str, int]) -> Move:
        data = await self.http.get(f"move/{name_or_id}")
        return Move(**data)

    async def get_move_ailment(self, name_or_id: Union[str, int]) -> MoveAilment:
        data = await self.http.get(f"move-ailment/{name_or_id}")
        return MoveAilment(**data)

    async def get_move_battle_style(
        self, name_or_id: Union[str, int]
    ) -> MoveBatteStyle:
        data = await self.http.get(f"move-battle-style/{name_or_id}")
        return MoveBatteStyle(**data)

    async def get_move_category(self, name_or_id: Union[str, int]) -> MoveCategory:
        data = await self.http.get(f"move-category/{name_or_id}")
        return MoveCategory(**data)

    async def get_move_damage_class(
        self, name_or_id: Union[str, int]
    ) -> MoveDamageClass:
        data = await self.http.get(f"move-damage-class/{name_or_id}")
        return MoveDamageClass(**data)

    async def get_move_learn_method(
        self, name_or_id: Union[str, int]
    ) -> MoveLearnMethod:
        data = await self.http.get(f"move-learn-method/{name_or_id}")
        return MoveLearnMethod(**data)

    async def get_move_target(self, name_or_id: Union[str, int]) -> MoveTarget:
        data = await self.http.get(f"move-target/{name_or_id}")
        return MoveTarget(**data)

    async def get_nature(self, name_or_id: Union[str, int]) -> Nature:
        data = await self.http.get(f"nature/{name_or_id}")
        return Nature(**data)

    async def get_pal_park_area(self, name_or_id: Union[str, int]) -> PalParkArea:
        data = await self.http.get(f"pal-park-area/{name_or_id}")
        return PalParkArea(**data)

    async def get_pokeathlon_stat(self, name_or_id: Union[str, int]) -> PokeathlonStat:
        data = await self.http.get(f"pokeathlon-stat/{name_or_id}")
        return PokeathlonStat(**data)

    async def get_pokedex(self, name_or_id: Union[str, int]) -> Pokedex:
        data = await self.http.get(f"pokedex/{name_or_id}")
        return Pokedex(**data)

    async def get_pokemon(self, name_or_id: Union[str, int]) -> Pokemon:
        data = await self.http.get(f"pokemon/{name_or_id}")
        data["location_area_encounters"] = await self.http.get(
            f"pokemon/{data['id']}/encounters"
        )
        return Pokemon(**data)

    async def get_pokemon_color(self, name_or_id: Union[str, int]) -> PokemonColor:
        data = await self.http.get(f"pokemon-color/{name_or_id}")
        return PokemonColor(**data)

    async def get_pokemon_form(self, name_or_id: Union[str, int]) -> PokemonForm:
        data = await self.http.get(f"pokemon-form/{name_or_id}")
        return PokemonForm(**data)

    async def get_pokemon_habitat(self, name_or_id: Union[str, int]) -> PokemonHabitat:
        data = await self.http.get(f"pokemon-habitat/{name_or_id}")
        return PokemonHabitat(**data)

    async def get_pokemon_shape(self, name_or_id: Union[str, int]) -> PokemonShape:
        data = await self.http.get(f"pokemon-shape/{name_or_id}")
        return PokemonShape(**data)

    async def get_pokemon_species(self, name_or_id: Union[str, int]) -> PokemonSpecies:
        data = await self.http.get(f"pokemon-species/{name_or_id}")
        return PokemonSpecies(**data)

    async def get_region(self, name_or_id: Union[str, int]) -> Region:
        data = await self.http.get(f"region/{name_or_id}")
        return Region(**data)

    async def get_stat(self, name_or_id: Union[str, int]) -> Stat:
        data = await self.http.get(f"stat/{name_or_id}")
        return Stat(**data)

    async def get_super_contest_effect(
        self, name_or_id: Union[str, int]
    ) -> SuperContestEffect:
        data = await self.http.get(f"super-contest-effect/{name_or_id}")
        return SuperContestEffect(**data)

    async def get_type(self, name_or_id: Union[str, int]) -> NaturalGiftType:
        data = await self.http.get(f"type/{name_or_id}")
        return NaturalGiftType(**data)

    async def get_version(self, name_or_id: Union[str, int]) -> Version:
        data = await self.http.get(f"version/{name_or_id}")
        return Version(**data)

    async def get_version_group(self, name_or_id: Union[str, int]) -> VersionGroup:
        data = await self.http.get(f"version-group/{name_or_id}")
        return VersionGroup(**data)



import pytest

import aiopoke


@pytest.mark.asyncio
async def test_cache(client: aiopoke.AiopokeClient):
    pokemon = await client.get_pokemon(5)
    encounter_condition = await client.get_encounter_condition(2)

    cache = client._cache
    assert cache.has(pokemon)
    assert cache.has(encounter_condition)

    move = await client.get_move(1)
    contest_effect = await move.contest_effect.fetch()

    assert isinstance(contest_effect, aiopoke.ContestEffect)



from typing import Any, Dict, List

from aiopoke.objects.utility.common_models import Name, NamedResource


class Language(NamedResource):
    official: bool
    iso3166: str
    iso639: str
    names: List["Name"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        official: bool,
        iso3166: str,
        iso639: str,
        names: List[Dict[str, Any]]
    ):
        super().__init__(id=id, name=name)
        self.official = official
        self.iso3166 = iso3166
        self.iso639 = iso639
        self.names = [Name(**name) for name in names]



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility.common_models import NamedResource
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources.pokemon import PokemonSpecies


class Gender(NamedResource):
    pokemon_species_details: List["PokemonSpeciesGender"]
    required_for_evolution: List[MinimalResource["PokemonSpecies"]]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        pokemon_species_details: List[Dict[str, Any]],
        required_for_evolution: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.pokemon_species_details = [
            PokemonSpeciesGender(**pokemon_species_details)
            for pokemon_species_details in pokemon_species_details
        ]
        self.required_for_evolution = [
            MinimalResource(**required_for_evolution)
            for required_for_evolution in required_for_evolution
        ]


class PokemonSpeciesGender(Resource):
    pokemon_species: MinimalResource["PokemonSpecies"]
    rate: int

    def __init__(self, *, pokemon_species: Dict[str, Any], rate: int) -> None:
        self.pokemon_species = MinimalResource(**pokemon_species)
        self.rate = rate



from aiopoke.objects.resources.locations.location import Location
from aiopoke.objects.resources.locations.location_area import LocationArea
from aiopoke.objects.resources.locations.pal_park_area import PalParkArea
from aiopoke.objects.resources.locations.region import Region

__all__ = ("LocationArea", "Location", "PalParkArea", "Region")



from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import TYPE_CHECKING

from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import (
        EvolutionTrigger,
        Item,
        Location,
        Move,
        NaturalGiftType,
        PokemonSpecies,
    )


class EvolutionChain(Resource):
    baby_trigger_item: Optional[MinimalResource["Item"]]
    chain: "ChainLink"
    id: int

    def __init__(
        self,
        *,
        baby_trigger_item: Optional[Dict[str, Any]],
        chain: Dict[str, Any],
        id: int,
    ) -> None:
        self.baby_trigger_item = (
            MinimalResource(**baby_trigger_item)
            if baby_trigger_item is not None
            else None
        )
        self.chain = ChainLink(**chain)
        self.id = id


class ChainLink(Resource):
    evolution_details: List["EvolutionDetail"]
    evolves_to: List["ChainLink"]
    is_baby: bool
    species: MinimalResource["PokemonSpecies"]

    def __init__(
        self,
        evolution_details: List[Dict[str, Any]],
        evolves_to: List[Dict[str, Any]],
        is_baby: bool,
        species: Dict[str, Any],
    ) -> None:
        self.evolution_details = [
            EvolutionDetail(**evolution_detail)
            for evolution_detail in evolution_details
        ]
        self.evolves_to = [ChainLink(**evolve_to) for evolve_to in evolves_to]
        self.is_baby = is_baby
        self.species = MinimalResource(**species)


class EvolutionDetail(Resource):
    gender: Optional[int]
    held_item: Optional[MinimalResource["Item"]]
    item: Optional[MinimalResource["Item"]]
    known_move: Optional[MinimalResource["Move"]]
    known_move_type: Optional[MinimalResource["NaturalGiftType"]]
    location: Optional[MinimalResource["Location"]]
    min_affection: Optional[int]
    min_beauty: Optional[int]
    min_happiness: Optional[int]
    min_level: int
    needs_overworld_rain: Optional[bool]
    party_species: Optional[MinimalResource["PokemonSpecies"]]
    party_type: Optional[MinimalResource["NaturalGiftType"]]
    relative_physical_stats: Optional[int]
    time_of_day: str
    trade_species: Optional[MinimalResource["PokemonSpecies"]]
    trigger: MinimalResource["EvolutionTrigger"]
    turn_upside_down: bool

    def __init__(
        self,
        *,
        gender: Optional[int],
        held_item: Optional[Dict[str, Any]],
        item: Optional[Dict[str, Any]],
        known_move: Optional[Dict[str, Any]],
        known_move_type: Optional[Dict[str, Any]],
        location: Optional[Dict[str, Any]],
        min_affection: Optional[int],
        min_beauty: Optional[int],
        min_happiness: Optional[int],
        min_level: int,
        needs_overworld_rain: Optional[bool],
        party_species: Optional[Dict[str, Any]],
        party_type: Optional[Dict[str, Any]],
        relative_physical_stats: Optional[int],
        time_of_day: str,
        trade_species: Optional[Dict[str, Any]],
        trigger: Dict[str, Any],
        turn_upside_down: bool,
    ):
        self.gender = gender
        self.held_item = MinimalResource(**held_item) if held_item is not None else None
        self.item = MinimalResource(**item) if item is not None else None
        self.known_move = (
            MinimalResource(**known_move) if known_move is not None else None
        )
        self.known_move_type = (
            MinimalResource(**known_move_type) if known_move_type is not None else None
        )
        self.location = MinimalResource(**location) if location is not None else None
        self.min_affection = min_affection
        self.min_beauty = min_beauty
        self.min_happiness = min_happiness
        self.min_level = min_level
        self.needs_overworld_rain = needs_overworld_rain
        self.party_species = (
            MinimalResource(**party_species) if party_species is not None else None
        )
        self.party_type = (
            MinimalResource(**party_type) if party_type is not None else None
        )
        self.relative_physical_stats = relative_physical_stats
        self.time_of_day = time_of_day
        self.trade_species = (
            MinimalResource(**trade_species) if trade_species is not None else None
        )
        self.trigger = MinimalResource(**trigger)
        self.turn_upside_down = turn_upside_down



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility import Description, Name, NamedResource
from aiopoke.utils.minimal_resources import MinimalResource

if TYPE_CHECKING:
    from aiopoke.objects.resources import Item


class ItemAttribute(NamedResource):
    descriptions: List["Description"]
    items: List[MinimalResource["Item"]]
    names: List["Name"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        descriptions: List[Dict[str, Any]],
        items: List[Dict[str, Any]],
        names: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.descriptions = [Description(**description) for description in descriptions]
        self.items = [MinimalResource(**item) for item in items]
        self.names = [Name(**name) for name in names]



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.resources.games.version import Version
from aiopoke.objects.utility import NamedResource
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import MoveLearnMethod, Pokedex, Region


class VersionGroup(NamedResource):
    generation: MinimalResource["Region"]
    move_learn_methods: List[MinimalResource["MoveLearnMethod"]]
    order: int
    pokedexes: List[MinimalResource["Pokedex"]]
    regions: List[MinimalResource["Region"]]
    versions: List[MinimalResource["Version"]]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        generation: Dict[str, Any],
        move_learn_methods: List[Dict[str, Any]],
        order: int,
        pokedexes: List[Dict[str, Any]],
        regions: List[Dict[str, Any]],
        versions: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.generation = MinimalResource(**generation)
        self.move_learn_methods = [
            MinimalResource(**move_learn_method)
            for move_learn_method in move_learn_methods
        ]
        self.order = order
        self.pokedexes = [MinimalResource(**pokedex) for pokedex in pokedexes]
        self.regions = [MinimalResource(**region) for region in regions]
        self.versions = [MinimalResource(**version) for version in versions]


class VersionGroupDetail(Resource):
    level_learned_at: int
    move_learn_method: MinimalResource["MoveLearnMethod"]
    version_group: MinimalResource["VersionGroup"]

    def __init__(
        self,
        *,
        level_learned_at: int,
        move_learn_method: Dict[str, Any],
        version_group: Dict[str, Any],
    ) -> None:
        self.level_learned_at = level_learned_at
        self.move_learn_method = MinimalResource(**move_learn_method)
        self.version_group = MinimalResource(**version_group)



from dataclasses import field
from typing import Any
from typing import Callable
from typing import Coroutine
from typing import Dict
from typing import Generic
from typing import Optional
from typing import TYPE_CHECKING
from typing import TypeVar
from typing import Union

from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.aiopoke_client import AiopokeClient

T = TypeVar("T")


class Url(Resource, Generic[T]):
    url: str
    id: int
    endpoint: str = field(repr=False)

    _client: Optional["AiopokeClient"] = field(default=None, repr=False)
    _build_map: Dict[
        str, Callable[[Union[str, int]], Coroutine[Any, Any, Any]]
    ] = field(repr=False)

    def __init__(self, *, url: str) -> None:
        self.url = url

        self.id = int(self.url.split("/")[-2])
        self.endpoint = self.url.split("/")[-3]

    @property
    def client(self) -> Optional["AiopokeClient"]:
        if hasattr(self, "_client"):
            return self._client

        return None

    @classmethod
    def link(cls, client):
        cls._client = client
        cls._build_map = {
            "ability": client.get_ability,
            "berry": client.get_berry,
            "berry-firmness": client.get_berry_firmness,
            "berry-flavor": client.get_berry_flavor,
            "characteristic": client.get_characteristic,
            "contest-effect": client.get_contest_effect,
            "contest-type": client.get_contest_type,
            "egg-group": client.get_egg_group,
            "encounter-condition": client.get_encounter_condition,
            "encounter-condition-value": client.get_encounter_condition_value,
            "encounter-method": client.get_encounter_method,
            "evolution-chain": client.get_evolution_chain,
            "evolution-trigger": client.get_evolution_trigger,
            "gender": client.get_gender,
            "generation": client.get_generation,
            "growth-rate": client.get_growth_rate,
            "item": client.get_item,
            "item-attribute": client.get_item_attribute,
            "item-category": client.get_item_category,
            "item-fling-effect": client.get_item_fling_effect,
            "item-pocket": client.get_item_pocket,
            "language": client.get_language,
            "location": client.get_location,
            "location-area": client.get_location_area,
            "machine": client.get_machine,
            "move": client.get_move,
            "move-ailment": client.get_move_ailment,
            "move-battle-style": client.get_move_battle_style,
            "move-category": client.get_move_category,
            "move-damage-class": client.get_move_damage_class,
            "move-learn-method": client.get_move_learn_method,
            "move-target": client.get_move_target,
            "nature": client.get_nature,
            "pal-park-area": client.get_pal_park_area,
            "pokeathlon-stat": client.get_pokeathlon_stat,
            "pokedex": client.get_pokedex,
            "pokemon": client.get_pokemon,
            "pokemon-color": client.get_pokemon_color,
            "pokemon-form": client.get_pokemon_form,
            "pokemon-habitat": client.get_pokemon_habitat,
            "pokemon-shape": client.get_pokemon_shape,
            "pokemon-species": client.get_pokemon_species,
            "region": client.get_region,
            "stat": client.get_stat,
            "super-contest-effect": client.get_super_contest_effect,
            "type": client.get_type,
            "version": client.get_version,
            "version-group": client.get_version_group,
        }

    async def fetch(self, *, client: Optional["AiopokeClient"] = None) -> T:
        client = self.client or client
        if client is None:
            raise ValueError(
                "A client must be provided, if you create your own instances of this class"
            )

        obj: T = await self._build_map[self.endpoint](self.id)
        return obj


class MinimalResource(Url[T]):
    name: str
    url: str = field(repr=False)
    id: int
    endpoint: str = field(repr=False)

    def __init__(self, *, name: str, url: str) -> None:
        super().__init__(url=url)
        self.name = name



from aiopoke.utils.resource import Resource


class NamedResource(Resource):
    """A resource with a name and id"""

    name: str
    id: int

    def __init__(self, *, name: str, id: int):
        self.name = name
        self.id = id

    def __eq__(self, other: object) -> bool:
        return (
            isinstance(other, self.__class__)
            and other.name == self.name
            and other.id == self.id
        )



from aiopoke.objects.resources.berries import Berry
from aiopoke.objects.resources.berries import BerryFirmness
from aiopoke.objects.resources.berries import BerryFlavor
from aiopoke.objects.resources.contests import ContestEffect
from aiopoke.objects.resources.contests import ContestType
from aiopoke.objects.resources.contests import SuperContestEffect
from aiopoke.objects.resources.encounters import EncounterCondition
from aiopoke.objects.resources.encounters import EncounterConditionValue
from aiopoke.objects.resources.encounters import EncounterMethod
from aiopoke.objects.resources.evolutions import EvolutionChain
from aiopoke.objects.resources.evolutions import EvolutionTrigger
from aiopoke.objects.resources.games import Generation
from aiopoke.objects.resources.games import Pokedex
from aiopoke.objects.resources.games import Version
from aiopoke.objects.resources.games import VersionGroup
from aiopoke.objects.resources.items import Item
from aiopoke.objects.resources.items import ItemAttribute
from aiopoke.objects.resources.items import ItemCategory
from aiopoke.objects.resources.items import ItemFlingEffect
from aiopoke.objects.resources.items import ItemPocket
from aiopoke.objects.resources.locations import Location
from aiopoke.objects.resources.locations import LocationArea
from aiopoke.objects.resources.locations import PalParkArea
from aiopoke.objects.resources.locations import Region
from aiopoke.objects.resources.machines import Machine
from aiopoke.objects.resources.moves import Move
from aiopoke.objects.resources.moves import MoveAilment
from aiopoke.objects.resources.moves import MoveBatteStyle
from aiopoke.objects.resources.moves import MoveCategory
from aiopoke.objects.resources.moves import MoveDamageClass
from aiopoke.objects.resources.moves import MoveLearnMethod
from aiopoke.objects.resources.moves import MoveTarget
from aiopoke.objects.resources.pokemon import Ability
from aiopoke.objects.resources.pokemon import Characteristic
from aiopoke.objects.resources.pokemon import EggGroup
from aiopoke.objects.resources.pokemon import Gender
from aiopoke.objects.resources.pokemon import GrowthRate
from aiopoke.objects.resources.pokemon import NaturalGiftType
from aiopoke.objects.resources.pokemon import Nature
from aiopoke.objects.resources.pokemon import PokeathlonStat
from aiopoke.objects.resources.pokemon import Pokemon
from aiopoke.objects.resources.pokemon import PokemonColor
from aiopoke.objects.resources.pokemon import PokemonForm
from aiopoke.objects.resources.pokemon import PokemonHabitat
from aiopoke.objects.resources.pokemon import PokemonShape
from aiopoke.objects.resources.pokemon import PokemonSpecies
from aiopoke.objects.resources.pokemon import Stat

__all__ = (
    "Berry",
    "BerryFirmness",
    "BerryFlavor",
    "ContestEffect",
    "ContestType",
    "SuperContestEffect",
    "EncounterCondition",
    "EncounterConditionValue",
    "EncounterMethod",
    "EvolutionChain",
    "EvolutionTrigger",
    "Generation",
    "Pokedex",
    "VersionGroup",
    "Version",
    "ItemAttribute",
    "ItemCategory",
    "ItemFlingEffect",
    "ItemPocket",
    "Item",
    "LocationArea",
    "Location",
    "PalParkArea",
    "Region",
    "Machine",
    "MoveAilment",
    "MoveBatteStyle",
    "MoveCategory",
    "MoveDamageClass",
    "MoveLearnMethod",
    "MoveTarget",
    "Move",
    "Ability",
    "Characteristic",
    "EggGroup",
    "Gender",
    "GrowthRate",
    "NaturalGiftType",
    "Nature",
    "PokeathlonStat",
    "PokemonColor",
    "PokemonForm",
    "PokemonHabitat",
    "PokemonShape",
    "PokemonSpecies",
    "Pokemon",
    "Stat",
)



from aiopoke.objects.utility.common_models.description import Description
from aiopoke.objects.utility.common_models.effect import Effect
from aiopoke.objects.utility.common_models.encounter import Encounter
from aiopoke.objects.utility.common_models.flavor_text import FlavorText
from aiopoke.objects.utility.common_models.generation_game_index import (
    GenerationGameIndex,
)
from aiopoke.objects.utility.common_models.machine_version_detail import (
    MachineVersionDetail,
)
from aiopoke.objects.utility.common_models.name import Name
from aiopoke.objects.utility.common_models.named_resource import NamedResource
from aiopoke.objects.utility.common_models.sprite import Sprite
from aiopoke.objects.utility.common_models.sprites import Sprites
from aiopoke.objects.utility.common_models.verbose_effect import VerboseEffect
from aiopoke.objects.utility.common_models.version_encounter_detail import (
    VersionEncounterDetail,
)
from aiopoke.objects.utility.common_models.version_game_index import VersionGameIndex
from aiopoke.objects.utility.common_models.version_group_flavor_text import (
    VersionGroupFlavorText,
)

__all__ = (
    "Description",
    "Effect",
    "Encounter",
    "FlavorText",
    "GenerationGameIndex",
    "MachineVersionDetail",
    "Name",
    "NamedResource",
    "Sprite",
    "Sprites",
    "VerboseEffect",
    "VersionEncounterDetail",
    "VersionGameIndex",
    "VersionGroupFlavorText",
)



import pytest

import aiopoke


@pytest.mark.asyncio
async def test_client(client: aiopoke.AiopokeClient):
    berry = await client.get_berry(1)
    berry_flavor = await berry.flavors[0].flavor.fetch()

    assert hasattr(berry, "name")
    assert hasattr(berry, "id")
    assert isinstance(berry_flavor, aiopoke.BerryFlavor)



from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import TYPE_CHECKING

from aiopoke.objects.utility import GenerationGameIndex
from aiopoke.objects.utility import Name
from aiopoke.objects.utility import NamedResource
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import Generation, Move, MoveDamageClass
    from aiopoke.objects.resources.pokemon import Pokemon


class NaturalGiftType(NamedResource):
    damage_relations: "TypeRelations"
    game_indices: List["GenerationGameIndex"]
    generation: MinimalResource["Generation"]
    move_damage_class: Optional[MinimalResource["MoveDamageClass"]]
    moves: List[MinimalResource["Move"]]
    names: List["Name"]
    past_damage_relations: List["PastTypeRelation"]
    pokemon: List["TypePokemon"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        damage_relations: Dict[str, Any],
        game_indices: List[Dict[str, Any]],
        generation: Dict[str, Any],
        move_damage_class: Optional[Dict[str, Any]],
        moves: List[Dict[str, Any]],
        names: List[Dict[str, Any]],
        past_damage_relations: List[Dict[str, Any]],
        pokemon: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.damage_relations = TypeRelations(**damage_relations)
        self.game_indices = [
            GenerationGameIndex(**game_index) for game_index in game_indices
        ]
        self.generation = MinimalResource(**generation)
        self.move_damage_class = (
            MinimalResource(**move_damage_class)
            if move_damage_class is not None
            else None
        )
        self.moves = [MinimalResource(**move) for move in moves]
        self.names = [Name(**name) for name in names]
        self.past_damage_relations = [
            PastTypeRelation(**past_damage_relation)
            for past_damage_relation in past_damage_relations
        ]
        self.pokemon = [TypePokemon(**pokemon) for pokemon in pokemon]


class TypeRelations(Resource):
    double_damage_from: List[MinimalResource["NaturalGiftType"]]
    double_damage_to: List[MinimalResource["NaturalGiftType"]]
    half_damage_from: List[MinimalResource["NaturalGiftType"]]
    half_damage_to: List[MinimalResource["NaturalGiftType"]]
    no_damage_from: List[MinimalResource["NaturalGiftType"]]
    no_damage_to: List[MinimalResource["NaturalGiftType"]]

    def __init__(
        self,
        *,
        double_damage_from: List[Dict[str, Any]],
        double_damage_to: List[Dict[str, Any]],
        half_damage_from: List[Dict[str, Any]],
        half_damage_to: List[Dict[str, Any]],
        no_damage_from: List[Dict[str, Any]],
        no_damage_to: List[Dict[str, Any]],
    ) -> None:
        self.double_damage_from = [
            MinimalResource(**type_data) for type_data in double_damage_from
        ]
        self.double_damage_to = [
            MinimalResource(**type_data) for type_data in double_damage_to
        ]
        self.half_damage_from = [
            MinimalResource(**type_data) for type_data in half_damage_from
        ]
        self.half_damage_to = [
            MinimalResource(**type_data) for type_data in half_damage_to
        ]
        self.no_damage_from = [
            MinimalResource(**type_data) for type_data in no_damage_from
        ]
        self.no_damage_to = [MinimalResource(**type_data) for type_data in no_damage_to]


class PastTypeRelation:
    damage_relations: "TypeRelations"
    generation: MinimalResource["Generation"]

    def __init__(
        self,
        *,
        damage_relations: Dict[str, Any],
        generation: Dict[str, Any],
    ) -> None:
        self.damage_relations = TypeRelations(**damage_relations)
        self.generation = MinimalResource(**generation)


class TypePokemon(Resource):
    slot: int
    pokemon: MinimalResource["Pokemon"]

    def __init__(self, *, slot: int, pokemon: Dict[str, Any]) -> None:
        self.slot = slot
        self.pokemon = MinimalResource(**pokemon)



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility import Description, NamedResource
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources.pokemon import PokemonSpecies


class GrowthRate(NamedResource):
    descriptions: List["Description"]
    formula: str
    levels: List["GrowthRateExperienceLevel"]
    pokemon_species: List[MinimalResource["PokemonSpecies"]]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        descriptions: List[Dict[str, Any]],
        formula: str,
        levels: List[Dict[str, Any]],
        pokemon_species: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.descriptions = [Description(**description) for description in descriptions]
        self.formula = formula
        self.levels = [GrowthRateExperienceLevel(**level) for level in levels]
        self.pokemon_species = [
            MinimalResource(**pokemon_species) for pokemon_species in pokemon_species
        ]


class GrowthRateExperienceLevel(Resource):
    level: int
    experience: int

    def __init__(self, *, level: int, experience: int) -> None:
        self.level = level
        self.experience = experience



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility.common_models import Name, NamedResource
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources.pokemon import Nature


class PokeathlonStat(NamedResource):
    affecting_natures: "NaturePokeathlonStatAffectSets"
    names: List["Name"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        affecting_natures: Dict[str, Any],
        names: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.affecting_natures = NaturePokeathlonStatAffectSets(**affecting_natures)
        self.names = [Name(**name) for name in names]


class NaturePokeathlonStatAffectSets(Resource):
    increase: List["NaturePokeathlonStatAffect"]
    decrease: List["NaturePokeathlonStatAffect"]

    def __init__(
        self, *, increase: List[Dict[str, Any]], decrease: List[Dict[str, Any]]
    ) -> None:
        self.increase = [
            NaturePokeathlonStatAffect(**increase_data) for increase_data in increase
        ]
        self.decrease = [
            NaturePokeathlonStatAffect(**decrease_data) for decrease_data in decrease
        ]


class NaturePokeathlonStatAffect(Resource):
    max_change: int
    nature: MinimalResource["Nature"]

    def __init__(
        self,
        *,
        max_change: int,
        nature: Dict[str, Any],
    ) -> None:
        self.max_change = max_change
        self.nature = MinimalResource(**nature)



from aiopoke.objects.resources.items.item import Item
from aiopoke.objects.resources.items.item_attribute import ItemAttribute
from aiopoke.objects.resources.items.item_category import ItemCategory
from aiopoke.objects.resources.items.item_fling_effect import ItemFlingEffect
from aiopoke.objects.resources.items.item_pocket import ItemPocket

__all__ = (
    "ItemAttribute",
    "ItemCategory",
    "ItemCategory",
    "ItemFlingEffect",
    "ItemPocket",
    "Item",
)



from typing import Any, Dict, List

from aiopoke.objects.utility.common_models import Name, NamedResource


class EncounterMethod(NamedResource):
    order: int
    names: List["Name"]

    def __init__(
        self, *, id: int, name: str, order: int, names: List[Dict[str, Any]]
    ) -> None:
        super().__init__(id=id, name=name)
        self.order = order
        self.names = [Name(**name) for name in names]



from typing import TYPE_CHECKING, Any, Dict

from aiopoke.utils.minimal_resources import MinimalResource, Url
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import Machine, VersionGroup


class MachineVersionDetail(Resource):
    machine: Url["Machine"]
    version_group: MinimalResource["VersionGroup"]

    def __init__(
        self,
        *,
        machine: Dict[str, Any],
        version_group: Dict[str, Any],
    ):
        self.machine = Url(**machine)
        self.version_group = MinimalResource(**version_group)



from typing import TYPE_CHECKING, Any, Dict

from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import Item, Move, VersionGroup


class Machine(Resource):
    id: int
    item: MinimalResource["Item"]
    move: MinimalResource["Move"]
    version_group: MinimalResource["VersionGroup"]

    def __init__(
        self,
        *,
        id: int,
        item: Dict[str, Any],
        move: Dict[str, Any],
        version_group: Dict[str, Any]
    ) -> None:
        self.id = id
        self.item = MinimalResource(**item)
        self.move = MinimalResource(**move)
        self.version_group = MinimalResource(**version_group)



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility import Name, NamedResource, VersionEncounterDetail
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import EncounterMethod, Pokemon, Version
    from aiopoke.objects.resources.locations import Location


class LocationArea(NamedResource):
    encounter_method_rates: List["EncounterMethodRate"]
    pokemon_encounters: List["PokemonEncounter"]
    location: MinimalResource["Location"]
    game_index: int
    names: List["Name"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        encounter_method_rates: List[Dict[str, Any]],
        pokemon_encounters: List[Dict[str, Any]],
        location: Dict[str, Any],
        game_index: int,
        names: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.encounter_method_rates = [
            EncounterMethodRate(**encounter_method_rate)
            for encounter_method_rate in encounter_method_rates
        ]
        self.pokemon_encounters = [
            PokemonEncounter(**pokemon_encounter)
            for pokemon_encounter in pokemon_encounters
        ]
        self.location = MinimalResource(**location)
        self.game_index = game_index
        self.names = [Name(**name) for name in names]


class PokemonEncounter(Resource):
    pokemon: MinimalResource["Pokemon"]
    version_details: List["VersionEncounterDetail"]

    def __init__(
        self,
        *,
        pokemon: Dict[str, Any],
        version_details: List[Dict[str, Any]],
    ) -> None:
        self.pokemon = MinimalResource(**pokemon)
        self.version_details = [
            VersionEncounterDetail(**version_encounter_detail)
            for version_encounter_detail in version_details
        ]


class EncounterMethodRate(Resource):
    encounter_method: MinimalResource["EncounterMethod"]
    version_details: List["EncounterVersionDetail"]

    def __init__(
        self,
        *,
        encounter_method: Dict[str, Any],
        version_details: List[Dict[str, Any]],
    ) -> None:
        self.encounter_method = MinimalResource(**encounter_method)
        self.version_details = [
            EncounterVersionDetail(**version_encounter_detail)
            for version_encounter_detail in version_details
        ]


class EncounterVersionDetail(Resource):
    rate: int
    version: MinimalResource["Version"]

    def __init__(
        self,
        *,
        rate: int,
        version: Dict[str, Any],
    ) -> None:
        self.rate = rate
        self.version = MinimalResource(**version)



from aiopoke.objects.resources.moves.move import Move
from aiopoke.objects.resources.moves.move_ailment import MoveAilment
from aiopoke.objects.resources.moves.move_battle_style import MoveBatteStyle
from aiopoke.objects.resources.moves.move_category import MoveCategory
from aiopoke.objects.resources.moves.move_damage_class import MoveDamageClass
from aiopoke.objects.resources.moves.move_learn_method import MoveLearnMethod
from aiopoke.objects.resources.moves.move_target import MoveTarget

__all__ = (
    "MoveAilment",
    "MoveBatteStyle",
    "MoveCategory",
    "MoveDamageClass",
    "MoveLearnMethod",
    "MoveTarget",
    "Move",
)



from typing import TYPE_CHECKING, Any, Dict, List

from aiopoke.objects.utility import Effect, NamedResource, VerboseEffect
from aiopoke.objects.utility.common_models import Name
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import Generation, VersionGroup
    from aiopoke.objects.resources.pokemon import Pokemon
    from aiopoke.objects.utility import Language


class Ability(NamedResource):
    effect_changes: List["AbilityEffectChange"]
    effect_entries: List["VerboseEffect"]
    flavor_text_entries: List["AbilityFlavorText"]
    generation: MinimalResource["Generation"]
    is_main_series: bool
    names: List["Name"]
    pokemon: List["AbilityPokemon"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        effect_changes: List[Dict[str, Any]],
        effect_entries: List[Dict[str, Any]],
        flavor_text_entries: List[Dict[str, Any]],
        generation: Dict[str, Any],
        is_main_series: bool,
        names: List[Dict[str, Any]],
        pokemon: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.effect_changes = [
            AbilityEffectChange(**effect_change) for effect_change in effect_changes
        ]
        self.effect_entries = [
            VerboseEffect(**effect_entry) for effect_entry in effect_entries
        ]
        self.flavor_text_entries = [
            AbilityFlavorText(**flavor_text_entry)
            for flavor_text_entry in flavor_text_entries
        ]
        self.generation = MinimalResource(**generation)
        self.is_main_series = is_main_series
        self.names = [Name(**name) for name in names]
        self.pokemon = [AbilityPokemon(**pokemon) for pokemon in pokemon]


class AbilityPokemon(Resource):
    is_hidden: bool
    slot: int
    pokemon: MinimalResource["Pokemon"]

    def __init__(self, *, is_hidden: bool, slot: int, pokemon: Dict[str, Any]) -> None:
        self.is_hidden = is_hidden
        self.slot = slot
        self.pokemon = MinimalResource(**pokemon)


class AbilityEffectChange(Resource):
    effect_entries: List["Effect"]
    version_group: MinimalResource["VersionGroup"]

    def __init__(
        self, *, effect_entries: List[Dict[str, Any]], version_group: Dict[str, Any]
    ) -> None:
        self.effect_entries = [
            Effect(**effect_entry) for effect_entry in effect_entries
        ]
        self.version_group = MinimalResource(**version_group)


class AbilityFlavorText(Resource):
    flavor_text: str
    language: MinimalResource["Language"]
    version_group: MinimalResource["VersionGroup"]

    def __init__(
        self,
        *,
        flavor_text: str,
        language: Dict[str, Any],
        version_group: Dict[str, Any],
    ) -> None:
        self.flavor_text = flavor_text
        self.language = MinimalResource(**language)
        self.version_group = MinimalResource(**version_group)



from dataclasses import dataclass


class UseDataclass(type):
    def __new__(cls, name, bases, clsdict):
        clsobj = super().__new__(cls, name, bases, clsdict)
        return dataclass(clsobj, init=False)  # type: ignore


class Resource(metaclass=UseDataclass):
    pass



from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import TYPE_CHECKING

from aiopoke.objects.utility import Description
from aiopoke.objects.utility import FlavorText
from aiopoke.objects.utility import Name
from aiopoke.objects.utility import NamedResource
from aiopoke.utils.minimal_resources import MinimalResource
from aiopoke.utils.minimal_resources import Url
from aiopoke.utils.resource import Resource

if TYPE_CHECKING:
    from aiopoke.objects.resources import (
        EvolutionChain,
        Generation,
        PalParkArea,
        Pokedex,
    )
    from aiopoke.objects.resources.pokemon import (
        EggGroup,
        GrowthRate,
        Pokemon,
        PokemonColor,
        PokemonHabitat,
        PokemonShape,
    )
    from aiopoke.objects.utility import Language


class PokemonSpecies(NamedResource):
    base_happiness: int
    capture_rate: int
    color: MinimalResource["PokemonColor"]
    egg_groups: List[MinimalResource["EggGroup"]]
    evolution_chain: Url["EvolutionChain"]
    evolves_from_species: Optional[MinimalResource["PokemonSpecies"]]
    flavor_text_entries: List["FlavorText"]
    form_descriptions: List["Description"]
    forms_switchable: bool
    gender_rate: int
    genera: List["Genus"]
    generation: MinimalResource["Generation"]
    growth_rate: MinimalResource["GrowthRate"]
    habitat: Optional[MinimalResource["PokemonHabitat"]]
    has_gender_differences: bool
    hatch_counter: int
    is_baby: bool
    is_legendary: bool
    is_mythical: bool
    order: int
    names: List["Name"]
    pal_park_encounters: List["PalParkEncounterArea"]
    pokedex_numbers: List["PokemonSpeciesDexEntry"]
    shape: Optional[MinimalResource["PokemonShape"]]
    varieties: List["PokemonSpeciesVariety"]

    def __init__(
        self,
        *,
        id: int,
        name: str,
        base_happiness: int,
        capture_rate: int,
        color: Dict[str, Any],
        egg_groups: List[Dict[str, Any]],
        evolution_chain: Dict[str, Any],
        evolves_from_species: Optional[Dict[str, Any]],
        flavor_text_entries: List[Dict[str, Any]],
        form_descriptions: List[Dict[str, Any]],
        forms_switchable: bool,
        gender_rate: int,
        genera: List[Dict[str, Any]],
        generation: Dict[str, Any],
        growth_rate: Dict[str, Any],
        habitat: Dict[str, Any],
        has_gender_differences: bool,
        hatch_counter: int,
        is_baby: bool,
        is_legendary: bool,
        is_mythical: bool,
        order: int,
        names: List[Dict[str, Any]],
        pal_park_encounters: List[Dict[str, Any]],
        pokedex_numbers: List[Dict[str, Any]],
        shape: Dict[str, Any],
        varieties: List[Dict[str, Any]],
    ) -> None:
        super().__init__(id=id, name=name)
        self.base_happiness = base_happiness
        self.capture_rate = capture_rate
        self.color = MinimalResource(**color)
        self.egg_groups = [MinimalResource(**egg_group) for egg_group in egg_groups]
        self.evolution_chain = Url(**evolution_chain)
        self.evolves_from_species = (
            MinimalResource(**evolves_from_species)
            if evolves_from_species is not None
            else None
        )
        self.flavor_text_entries = [
            FlavorText(**flavor_text_entry) for flavor_text_entry in flavor_text_entries
        ]
        self.form_descriptions = [
            Description(**form_description) for form_description in form_descriptions
        ]
        self.forms_switchable = forms_switchable
        self.gender_rate = gender_rate
        self.genera = [Genus(**genus_data) for genus_data in genera]
        self.generation = MinimalResource(**generation)
        self.growth_rate = MinimalResource(**growth_rate)
        self.habitat = MinimalResource(**habitat) if habitat is not None else None
        self.has_gender_differences = has_gender_differences
        self.hatch_counter = hatch_counter
        self.is_baby = is_baby
        self.is_legendary = is_legendary
        self.is_mythical = is_mythical
        self.order = order
        self.names = [Name(**name) for name in names]
        self.pal_park_encounters = [
            PalParkEncounterArea(**pal_park_encounter)
            for pal_park_encounter in pal_park_encounters
        ]
        self.pokedex_numbers = [
            PokemonSpeciesDexEntry(**pokedex_number)
            for pokedex_number in pokedex_numbers
        ]
        self.shape = MinimalResource(**shape) if shape is not None else None
        self.varieties = [PokemonSpeciesVariety(**variety) for variety in varieties]


class Genus(Resource):
    genus: str
    language: MinimalResource["Language"]

    def __init__(self, genus: str, language: Dict[str, Any]) -> None:
        self.genus = genus
        self.language = MinimalResource(**language)


class PokemonSpeciesDexEntry(Resource):
    entry_number: int
    pokedex: MinimalResource["Pokedex"]

    def __init__(self, *, entry_number: int, pokedex: Dict[str, Any]) -> None:
        self.entry_number = entry_number
        self.pokedex = MinimalResource(**pokedex)


class PalParkEncounterArea(Resource):
    base_score: int
    rate: int
    area: MinimalResource["PalParkArea"]

    def __init__(self, *, base_score: int, rate: int, area: Dict[str, Any]) -> None:
        self.base_score = base_score
        self.rate = rate
        self.area = MinimalResource(**area)


class PokemonSpeciesVariety(Resource):
    is_default: bool
    pokemon: MinimalResource["Pokemon"]

    def __init__(self, *, is_default: bool, pokemon: Dict[str, Any]) -> None:
        self.is_default = is_default
        self.pokemon = MinimalResource(**pokemon)



from aiopoke.objects.resources.games.generation import Generation
from aiopoke.objects.resources.games.pokedex import Pokedex
from aiopoke.objects.resources.games.version import Version
from aiopoke.objects.resources.games.version_group import VersionGroup

__all__ = ("Generation", "Pokedex", "VersionGroup", "Version")
